<?php
/**
 * Handler: for managing Wordpress sidebars
 */
namespace EwTools\Handler;

/**
 * Sidebar
 * @todo Make ORM-object of this class
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\Sidebars 0.1
 */
class Sidebars {

	/**
	 * Contains all registered sidebars
	 * @var array
	 */
	protected static $sidebars = [];
	
	/**
	 * Function to add a sidebar to the static list of sidebars
	 *
	 * @param string $id
	 * @param array $options
	 *
	 * @since 0.1
	 * @return void
	 */
	protected static function add( $id, $options ) {
		static::$sidebars[$id] = $options;
	}
	
	/**
	 * Function to check if a sidebar exists in the collection
	 *
	 * @param string $id
     *
	 * @uses is_active_sidebar()
     *
	 * @since 0.1
	 * @return bool
	 */
	public static function exist( $id ) {
		return array_key_exists( $id, static::$sidebars ) && is_active_sidebar( $id );
	}
	
	/**
	 * Function to get a specific sidebar
	 *
	 * @param string $id
	 *
	 * @return array|false
	 */
	public static function get( $id ) {
		return static::exist( $id ) ? static::$sidebars[$id] : false;
	}
	
	/**
	 * Function for retrieving the sidebar content set in admin
	 *
	 * @param string $id
	 *
     * @uses dynamic_sidebar()
     *
	 * @since 0.1
	 * @return string
	 */
	public static function getContent( $id ) {
		$sidebar_content = '';

		if ( static::exist( $id ) ) {
			ob_start();
			dynamic_sidebar( $id );
			$sidebar_content = ob_get_contents();
			ob_clean();
		}

		return $sidebar_content;
	}
	
	/**
	 * Function to register our sidebars with our own defaults
	 *
	 * @param string $id
	 * @param array  $options
	 *
     * @uses register_sidebar()
     *
	 * @since 0.1
	 * @return void
	 */
	public static function register( $id, array $options ) {
		// Add new defaults
		$sidebar_options = array_replace_recursive([
			'name'          => 'No name',
			'description'   => 'No description',
			'id'            => $id,
			'class'         => '',
			'before_widget' => '',
			'after_widget'  => '',
			'before_title'  => '',
			'after_title'   => '',
		], $options);

		// Add translation options
		$sidebar_options['name']        = __($sidebar_options['name'], $id.'-name');
		$sidebar_options['description'] = __($sidebar_options['description'], $id.'-description');

		static::add( $id, $sidebar_options);
		register_sidebar( $sidebar_options );
	}

	/**
	 * Function for rendering the sidebar content set in admin
	 *
	 * @param string $id
	 *
     * @uses dynamic_sidebar()
     *
     * @since 0.1
	 * @return void
	 */
	public static function render( $id ) {
		if ( static::exist( $id ) ) {
			dynamic_sidebar( $id );
		}
	}
}