<?php
/**
 * Trait: for adding macro-ability to the affected class
 */
namespace EwTools\Support\Traits;

use Closure;
use BadMethodCallException;

/**
 * Macroable
 *
 * @link    http://infomaker.se
 * @package EwTools\Support\Traits
 * @since   EwTools\Support\Traits\Macroable 0.1
 */
trait Macroable {
	
	/**
	 * The registered string macros.
	 * @var array
	 */
	protected static $macros = [];
	
	/**
	 * Register a custom macro.
	 *
	 * @param  string   $name
	 * @param  callable $macro
	 *
	 * @since 0.1
	 * @return void
	 */
	public static function macro( $name, callable $macro ) {
		static::$macros[ $name ] = $macro;
	}
	
	/**
	 * Checks if macro is registered.
	 *
	 * @param  string $name
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function hasMacro( $name ) {
		return isset( static::$macros[ $name ] );
	}
	
	/**
	 * Dynamically handle calls to the class.
	 *
	 * @param  string $method
	 * @param  array  $parameters
	 *
	 * @since 0.1
	 * @return mixed
	 * @throws \BadMethodCallException
	 */
	public static function __callStatic( $method, $parameters ) {
		if ( static::hasMacro( $method ) ) {
			if ( static::$macros[ $method ] instanceof Closure ) {
				return call_user_func_array( Closure::bind( static::$macros[ $method ], null, get_called_class() ), $parameters );
			} else {
				return call_user_func_array( static::$macros[ $method ], $parameters );
			}
		}
		
		throw new BadMethodCallException( "Method {$method} does not exist." );
	}
	
	/**
	 * Dynamically handle calls to the class.
	 *
	 * @param  string $method
	 * @param  array  $parameters
	 *
	 * @since 0.1
	 * @return mixed
	 * @throws \BadMethodCallException
	 */
	public function __call( $method, $parameters ) {
		if ( static::hasMacro( $method ) ) {
            if ( static::$macros[ $method ] instanceof Closure ) {
                return call_user_func_array( Closure::bind( static::$macros[ $method ], null, get_called_class() ), $parameters );
            } else {
                return call_user_func_array( static::$macros[ $method ], $parameters );
            }
		}
		
		throw new BadMethodCallException( "Method {$method} does not exist." );
	}
}
